
TOOL_NAME = "Automax Selected Device"

----------------------------------------------------------------------
--preferences
----------------------------------------------------------------------
local options = renoise.Document.create {
  auto_max_enabled = false,
  auto_max_mixer_device_enabled = false,
  last_selected_mode = "auto_max_flag" --default 
}
renoise.tool().preferences = options
--options files need to be accessed via 'options.variable.value'
--i.e: 
--local auto_max_boolean = options.auto_max_enabled.value

--"keybinding
-------------
renoise.tool():add_keybinding {
  name = "Global:Tools:`AMX` Minimise All Devices Current Track",  
  invoke = function() collapse_all_devices()
  end  
}

--"keybinding
-------------
renoise.tool():add_keybinding {
  name = "Global:Tools:`AMX` Toggle Tool Bypass",  
  invoke = function() toggle_tool_bypass()
  end  
}
---------------------------------
--menu registration on DSP Device
---------------------------------
--menu check/tick for user to enable tool
-----------------------------
renoise.tool():add_menu_entry {
  name = "DSP Device:Device Chain:Auto Maximize Selected Device:Single Operation",
  selected = function() return options.auto_max_enabled.value end,
  invoke = function() 
    options.auto_max_enabled.value = not options.auto_max_enabled.value
    options.auto_max_mixer_device_enabled.value = false
  end
}
--alternative mode check/tick
-----------------------------
renoise.tool():add_menu_entry {
  name = "DSP Device:Device Chain:Auto Maximize Selected Device:Collapse When Mixer Device Chosen",
  selected = function() return options.auto_max_mixer_device_enabled.value end,
  invoke = function() 
    options.auto_max_mixer_device_enabled.value = not options.auto_max_mixer_device_enabled.value
    options.auto_max_enabled.value = false
  end
}

--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end


-----------------------------
function toggle_tool_bypass()
----------------------------- 

  --toggle off
  if options.auto_max_enabled.value == true then
    options.auto_max_enabled.value = false
    --update flag
    options.last_selected_mode.value = "auto_max_flag"
    status("automax bypassed")
    return
  end
  --toggle off
  if options.auto_max_mixer_device_enabled.value == true then
    options.auto_max_mixer_device_enabled.value = false
    --update flag
    options.last_selected_mode.value = "mixer_device_flag"
    status("automax bypassed")
    return
  end

  --if here, both must be off so toggle on last selected mode on (checking the flag)
  if options.last_selected_mode.value == "auto_max_flag" then
    --set auto max on
    options.auto_max_enabled.value = true
    --run automax fn
    maximise_selected_dev_collapse_all_others()
    status("automax enabled")
  elseif options.last_selected_mode.value == "mixer_device_flag" then
    --set automax mixer on
    options.auto_max_mixer_device_enabled.value = true
    collapse_all_devs_except_mixer()
    status("automax mixer-device-mode enabled")
  end  
end

--keyboard shortcut fn
-------------------------------------------
--collapse all the devices in current track
-------------------------------------------
function collapse_all_devices()  
  --song
  local song = renoise.song()
  --selected track
  local track = song.selected_track
  --flag
  local all_collapsed = true
  --loop devices in track and collapse all
  for i = 2,#track.devices do
    if track:device(i).is_maximized == true then
      all_collapsed = false
    end
  end
  
  if all_collapsed == true then
    --loop devices in track and un-collapse all
    for i = 2,#track.devices do
      track:device(i).is_maximized = true
    end
  else
    --loop devices in track and collapse all
    for i = 2,#track.devices do
      track:device(i).is_maximized = false
    end
  end
end
----

----------------------------------------------------------------------
--functions that caollapse relevant non-selected devices
----------------------------------------------------------------------
----------------------------------------------------
function maximise_selected_dev_collapse_all_others()
----------------------------------------------------
  --song object
  local song = renoise.song()
  local selected_track = song.selected_track
  local selected_device_index = song.selected_track_device_index
  
  --a device selected?
  if (selected_device_index ~= 0) then    
    --from 2 so as not to try to minimize the mixer device
    for i = 2,#selected_track.devices do
      if (i == selected_device_index) then
        selected_track:device(i).is_maximized = true
      else
        selected_track:device(i).is_maximized = false 
      end
    end
  end
end

--function colapses all devices when mixer devices is selected
--otherwise expands any device you select
-----------------------------------------
function collapse_all_devs_except_mixer()
-----------------------------------------
  --song object
  local song = renoise.song()
  local selected_track = song.selected_track
  local selected_device_index = song.selected_track_device_index
 
  --mixer device selected so collapse all
  if selected_device_index == 1 then
    --loop from 2 so as not to try to minimize the mixer device
    for i = 2,#selected_track.devices do
      selected_track:device(i).is_maximized = false 
    end
  elseif selected_device_index > 1 then
    selected_track:device(selected_device_index).is_maximized = true
  end
end

--selected device notifier calls main()
---------------
function main()
---------------
  --single click operation enabled in menu
  if (options.auto_max_enabled.value) then 
    maximise_selected_dev_collapse_all_others()
  end
  --mixer device selection enabled in menu
  if (options.auto_max_mixer_device_enabled.value) then 
    collapse_all_devs_except_mixer()
  end
end

-----------
--notifiers
-----------
----------------
function start()
----------------  
  renoise.song().selected_track_observable:add_notifier(main) 
  renoise.song().selected_track_device_observable:add_notifier(main)
end  
--new song notifier makes sure tool starts with aeach new song
renoise.tool().app_new_document_observable:add_notifier(start)





